"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isSubscription = exports.isClass = exports.fetchIntrospectionSchema = exports.stripObject = exports.getUrl = exports.getSchemaQuery = exports.getSchemaMutation = exports.getSchemaTypeFields = exports.getParsedSchema = exports.config = exports.registryGet = exports.performSafeRequestGraphql = exports.stripTypenameDefault = exports.stripTypename = exports.performGqlSubscription = exports.performGqlMutation = exports.performGqlQuery = exports.performSafeRequestREST = exports.queryParams = exports.camelToKebab = exports.getApolloClient = exports.defaultRESTHeaders = void 0;
const tslib_1 = require("tslib");
const graphql_1 = require("graphql");
const parser_1 = require("graphql/language/parser");
const omit_deep_lodash_1 = tslib_1.__importDefault(require("omit-deep-lodash"));
const Registry_1 = tslib_1.__importDefault(require("./Registry"));
const UnexpectedException_1 = tslib_1.__importDefault(require("./models/Exceptions/UnexpectedException"));
const makeApolloClient_1 = tslib_1.__importDefault(require("./graphql/makeApolloClient"));
exports.defaultRESTHeaders = {
    'Accept': 'application/json',
    'Content-Type': 'application/json;charset=UTF-8',
};
function getApolloClient(providerName = 'default') {
    return config(providerName).apolloClient || makeApolloClient_1.default(providerName);
}
exports.getApolloClient = getApolloClient;
function camelToKebab(input) {
    return input.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
}
exports.camelToKebab = camelToKebab;
function queryParams(params, camelToKebabActive = true) {
    const str = [];
    for (const paramsKey in params)
        if (params.hasOwnProperty(paramsKey) && (params[paramsKey] || params[paramsKey] === 0)) {
            const normalized = camelToKebabActive ? camelToKebab(encodeURIComponent(paramsKey)) : encodeURIComponent(paramsKey);
            str.push(`${normalized}=${encodeURIComponent(params[paramsKey])}`);
        }
    return str.join('&');
}
exports.queryParams = queryParams;
function performSafeRequestREST(url, params = {}, method = 'get', opts = {}) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        let fullUrl = url;
        let body = {};
        if (method === 'get') {
            const queryEscaped = queryParams(params);
            if (queryEscaped) {
                fullUrl = `${fullUrl}?${queryEscaped}`;
            }
        }
        else {
            body = params;
        }
        return fetch(fullUrl, Object.assign({ method: method.toLowerCase(), headers: exports.defaultRESTHeaders, body: JSON.stringify(body) }, opts)).then(response => response.json());
    });
}
exports.performSafeRequestREST = performSafeRequestREST;
function performGqlQuery(query, variables, providerName = 'default') {
    return getApolloClient(providerName).query({
        query,
        variables,
    });
}
exports.performGqlQuery = performGqlQuery;
function performGqlMutation(mutation, variables, providerName = 'default') {
    return getApolloClient(providerName).mutate({
        mutation,
        variables,
    });
}
exports.performGqlMutation = performGqlMutation;
function performGqlSubscription(subscription, variables, providerName = 'default') {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return getApolloClient(providerName).subscribe({
            query: subscription,
            variables,
        });
    });
}
exports.performGqlSubscription = performGqlSubscription;
function stripTypename(obj) {
    return config().stripTypename ? config().stripTypename(obj) : stripTypenameDefault(obj);
}
exports.stripTypename = stripTypename;
function stripTypenameDefault(obj) {
    return omit_deep_lodash_1.default(obj, '__typename');
}
exports.stripTypenameDefault = stripTypenameDefault;
function performSafeRequestGraphql(query, variables = {}, providerName = 'default') {
    var _a, _b;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const operationDefinition = query.definitions.find(def => def.kind === 'OperationDefinition');
        const queryName = operationDefinition === null || operationDefinition === void 0 ? void 0 : operationDefinition.name.value;
        const isQuery = query.definitions.find(def => def.kind === 'OperationDefinition').operation === 'query';
        if (isQuery) {
            const firstField = (_b = (_a = operationDefinition === null || operationDefinition === void 0 ? void 0 : operationDefinition.selectionSet) === null || _a === void 0 ? void 0 : _a.selections) === null || _b === void 0 ? void 0 : _b.find(s => s.kind === 'Field');
            const name = firstField === null || firstField === void 0 ? void 0 : firstField.name.value;
            return performGqlQuery(query, stripTypename(variables), providerName)
                .then((value) => name ? value.data[name] : value.data);
        }
        const isSubscription = query.definitions.find(def => def.kind === 'OperationDefinition').operation === 'subscription';
        if (isSubscription) {
            return performGqlSubscription(query, stripTypename(variables), providerName);
        }
        return performGqlMutation(query, stripTypename(variables), providerName)
            .then((value) => value.data[queryName]);
    });
}
exports.performSafeRequestGraphql = performSafeRequestGraphql;
function registryGet(key) {
    return Registry_1.default.getInstance().get(key);
}
exports.registryGet = registryGet;
function config(name = 'default') {
    const defaultConfig = registryGet('Config');
    if (name === 'default') {
        return defaultConfig;
    }
    return defaultConfig.providers.find(config => config.name === name);
}
exports.config = config;
function getParsedSchema(configName = 'default') {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const configSchema = config(configName).schema;
        const configSchemaUrl = config(configName).schemaUrl;
        let schema = Registry_1.default.getInstance().get('schema');
        if (!schema && configSchema) {
            schema = configSchema;
        }
        else if (!schema && configSchemaUrl) {
            schema = yield fetchIntrospectionSchema(configSchemaUrl)
                .then(graphql_1.buildClientSchema.bind(null))
                .then(graphql_1.printSchema.bind(null))
                .then(parser_1.parse.bind(null));
            Registry_1.default.getInstance().set('schema', schema);
        }
        if (!schema) {
            throw new UnexpectedException_1.default('Configuration error: \'schema\' must be passed as a config key, e.g\n\nimport schema from \'raw-loader!@/../schema.graphql\';\n\n//...\n\nVue.use(VueOOP, {\n  //...,\n  schema,\n})\n\n;');
        }
        return schema;
    });
}
exports.getParsedSchema = getParsedSchema;
function getSchemaTypeFields(typeName, configName = 'default') {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return (yield getParsedSchema(configName))
            .definitions
            .find(def => (def.name || {}).value === typeName)
            .fields
            .map(f => f.name.value);
    });
}
exports.getSchemaTypeFields = getSchemaTypeFields;
function getSchemaMutation(mutationName, configName = 'default') {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return (yield getParsedSchema(configName))
            .definitions
            .find(def => (def.name || {}).value === 'Mutation')
            .fields
            .find(def => (def.name || {}).value === mutationName);
    });
}
exports.getSchemaMutation = getSchemaMutation;
function getSchemaQuery(queryName, configName = 'default') {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return (yield getParsedSchema(configName))
            .definitions
            .find(def => (def.name || {}).value === 'Query')
            .fields
            .find(def => (def.name || {}).value === queryName);
    });
}
exports.getSchemaQuery = getSchemaQuery;
function getUrl(_opts) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { url, params } = _opts;
        let resolvedUrl = url;
        if (typeof url === 'function') {
            resolvedUrl = yield url();
        }
        else {
            resolvedUrl = resolvedUrl.replace(/:([^\s\/?&]+)/gi, (_, m) => {
                const param = params[m];
                const hasParam = param !== undefined;
                if (hasParam) {
                    delete params[m];
                    return param;
                }
                return m;
            });
        }
        return resolvedUrl;
    });
}
exports.getUrl = getUrl;
function stripObject(obj) {
    return omit_deep_lodash_1.default(obj, 'loading');
}
exports.stripObject = stripObject;
function fetchIntrospectionSchema(url) {
    const body = JSON.stringify({ query: graphql_1.getIntrospectionQuery() });
    return fetch(url, {
        method: 'POST',
        headers: { 'Content-Type': 'application/json' },
        body
    })
        .then(res => res.json())
        .then(res => res.data);
}
exports.fetchIntrospectionSchema = fetchIntrospectionSchema;
exports.isClass = (fn) => /^\s*class/.test(fn.toString());
exports.isSubscription = (data) => data._subscriber;
//# sourceMappingURL=utils.js.map